@@ -36,10 +36,10 @@ static void add_menu_separator(void);
 GtkTooltips *tooltips;
 GtkWidget *window, *restartbutton, *stopbutton, *buttons_box, 
   *all_box, *savebutton, *sortbutton, *addbutton,
-  *filelist, *statusbar, *nextbutton, *pausebutton, *loadbutton,
+  *filelist, *statusbar, *nextbutton, *prevbutton, *pausebutton, *loadbutton,
   *minicheckbox, *randomcheckbox, *fileselector=NULL, *list_box,
   *scrollbar, *upbutton, *downbutton, *deletebutton, *popupmenu,
-  *up_item, *down_item, *delete_item, *info_item;
+  *up_item, *down_item, *delete_item, *info_item, *eventbox;
 GtkAdjustment *vadj;
 GtkAccelGroup *accel_group;
 
@@ -56,7 +56,7 @@ void update_window() {
   gchar statbuf[STRBUFSIZE];
 
   if (paused) {
-    snprintf(statbuf, STRBUFSIZE, "%s", STRPAUSED);
+    snprintf(statbuf, STRBUFSIZE, "%s", _("Paused"));
   } else if (playing) {
     if (attribute_mini) {
       /* Put the name of the song playing in the status bar. */
@@ -65,10 +65,10 @@ void update_window() {
     } else {
       /* Put the "Playing with" string in the status bar. */
       if (mode_show_pid) {
-	snprintf(statbuf, STRBUFSIZE, STRPLAYINGWITHPID, last_player,
+	snprintf(statbuf, STRBUFSIZE, _("Playing with %s (pid %d)"), last_player,
 		 childpid);
       } else {
-	snprintf(statbuf, STRBUFSIZE, STRPLAYINGWITH, last_player);
+	snprintf(statbuf, STRBUFSIZE, _("Playing with %s."), last_player);
       }
     }
 
@@ -78,7 +78,7 @@ void update_window() {
       gtk_clist_moveto(GTK_CLIST(filelist), last_played, 0, 0.5, 0.0);
     }
   } else {
-    snprintf(statbuf, STRBUFSIZE, STRSTOPPED);
+    snprintf(statbuf, STRBUFSIZE, _("Stopped."));
   }
 
   status(statbuf);
@@ -91,7 +91,7 @@ gint sigchld_timeout(gpointer data) {
   pick_next();
   if (!playing) { /* reached end of one time list? */
     donotstopflag = 1;
-    status(STRDONE);
+    status(_("All items played."));
   }
   return FALSE; /* don't do the timeout again */
 }
@@ -148,11 +148,11 @@ static GtkTargetEntry target_table[] = {
 
 void print_version() {
   g_print("GNU xhippo %s\n", VERSION);
-  g_print("Copyright (c) 1998-2000 Adam Sampson
-GNU xhippo comes with NO WARRANTY, to the extent permitted by law.
-You may redistribute copies of GNU xhippo under the terms of the
-GNU General Public License. For more information about these matters,
-see the file named COPYING.\n");
+  g_print("Copyright (c) 1998-2000 Adam Sampson\n"
+"GNU xhippo comes with NO WARRANTY, to the extent permitted by law.\n"
+"You may redistribute copies of GNU xhippo under the terms of the\n"
+"GNU General Public License. For more information about these matters,\n"
+"see the file named COPYING.\n");
 }
 
 static GtkWidget *add_button(gchar *label, gchar *tip, GtkSignalFunc callback,
@@ -207,10 +207,15 @@ int main(int argc, char **argv) {
 
   /* Check that HOME is available. */
   if (!getenv("HOME")) {
-    fprintf(stderr, STRSETHOME);
+    fprintf(stderr, _("You must set $HOME to your home directory.\n"));
     exit(20);
   }
 
+  /* Set up gettext. */
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
+
   /* Seed RNG. */
   srand(time(0));
 
@@ -280,14 +285,21 @@ int main(int argc, char **argv) {
 #endif
   gtk_widget_set_name(all_box, "all_box");
 
+  /* Create the event box for the status bar. */
+  eventbox = gtk_event_box_new();
+  gtk_signal_connect(GTK_OBJECT(eventbox), "button_press_event",
+		     GTK_SIGNAL_FUNC(handle_menu_popup), NULL);
+  gtk_box_pack_start(GTK_BOX(all_box), eventbox, FALSE, FALSE, 0);
+
   /* Create the status bar. */
   statusbar = gtk_statusbar_new();
-  gtk_box_pack_start(GTK_BOX(all_box), statusbar, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER(eventbox), statusbar);
   contextid = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "xhippo");
-  gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid, STRWELCOME);
+  gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid, _("Welcome to xhippo!"));
   gtk_widget_set_name(statusbar, "statusbar");
 
   gtk_widget_show(statusbar);
+  gtk_widget_show(eventbox);
 
   /* Create the upper buttons container. */
   buttons_box = gtk_hbox_new(FALSE, 0);
@@ -295,40 +307,47 @@ int main(int argc, char **argv) {
   gtk_widget_set_name(buttons_box, "buttons_box");
 
   /* Create the buttons and checkboxes in the upper bar. */
-  restartbutton = add_button(STRRESTART, STRRESTARTTIP, handle_restart,
-			      buttons_box, "restartbutton", 0);
+  restartbutton = add_button(_("Restart"), _("Restart the current song"), 
+                             handle_restart, buttons_box, "restartbutton", 0);
   gtk_widget_add_accelerator(restartbutton, "clicked", accel_group,
                              GDK_r, 0, GTK_ACCEL_VISIBLE);
 
-  stopbutton = add_button(STRSTOP, STRSTOPTIP, handle_stop,
+  stopbutton = add_button(_("Stop"), _("Stop playing"), handle_stop,
 			   buttons_box, "stopbutton", GDK_KP_Divide);
   gtk_widget_add_accelerator(stopbutton, "clicked", accel_group,
                              GDK_s, 0, GTK_ACCEL_VISIBLE);
 
-  pausebutton = add_button(STRPAUSE, STRPAUSETIP, handle_pause,
+  pausebutton = add_button(_("Pause"), _("Pause playing"), handle_pause,
 			    buttons_box, "pausebutton", GDK_KP_Add);
   gtk_widget_add_accelerator(pausebutton, "clicked", accel_group,
                              GDK_p, 0, GTK_ACCEL_VISIBLE);
 
-  nextbutton = add_button(STRNEXT, STRNEXTTIP, handle_next,
+  prevbutton = add_button(_("Prev"), _("Play the previous song"), handle_prev,
+			   buttons_box, "nextbutton", GDK_b);
+  
+  nextbutton = add_button(_("Next"), _("Play the next song"), handle_next,
 			   buttons_box, "nextbutton", GDK_KP_Subtract);
   gtk_widget_add_accelerator(nextbutton, "clicked", accel_group,
 			     GDK_KP_Multiply, 0, GTK_ACCEL_VISIBLE);
   gtk_widget_add_accelerator(nextbutton, "clicked", accel_group,
                              GDK_n, 0, GTK_ACCEL_VISIBLE);
   
-  minicheckbox = gtk_check_button_new_with_label(STRMINI);
+  minicheckbox = gtk_check_button_new_with_label(_("Mini"));
   gtk_signal_connect(GTK_OBJECT(minicheckbox), "toggled",
 		     GTK_SIGNAL_FUNC(handle_minitoggle), NULL);
   gtk_widget_set_name(minicheckbox, "minicheckbox");
   gtk_box_pack_start(GTK_BOX(buttons_box), minicheckbox, FALSE, FALSE, 0);
+  gtk_widget_add_accelerator(minicheckbox, "clicked", accel_group,
+                             GDK_quoteleft, 0, GTK_ACCEL_VISIBLE);
   gtk_widget_show(minicheckbox);
 
-  randomcheckbox = gtk_check_button_new_with_label(STRRANDOM);
+  randomcheckbox = gtk_check_button_new_with_label(_("Random"));
   gtk_signal_connect(GTK_OBJECT(randomcheckbox), "toggled",
 		     GTK_SIGNAL_FUNC(handle_randomtoggle), NULL);
   gtk_widget_set_name(randomcheckbox, "randomcheckbox");
   gtk_box_pack_start(GTK_BOX(buttons_box), randomcheckbox, FALSE, FALSE, 0);
+  gtk_widget_add_accelerator(randomcheckbox, "clicked", accel_group,
+                             GDK_h, 0, GTK_ACCEL_VISIBLE);
   gtk_widget_show(randomcheckbox);
 
   /* Show the buttons boxes. */
@@ -341,18 +360,25 @@ int main(int argc, char **argv) {
 
   /* Create the popup menu. */
   popupmenu = gtk_menu_new();
-  info_item = add_menu_item(STRINFO, handle_menu_info, 0);
-  up_item = add_menu_item(STRUP, handle_menu_up, 0);
-  down_item = add_menu_item(STRDOWN, handle_menu_down, 0);
-  delete_item = add_menu_item(STRDELETE, handle_menu_delete, 0);
+  info_item = add_menu_item(_("Song information"), handle_menu_info, 0);
+  up_item = add_menu_item(_("Move song up"), handle_menu_up, 0);
+  down_item = add_menu_item(_("Move song down"), handle_menu_down, 0);
+  delete_item = add_menu_item(_("Remove song"), handle_menu_delete, 0);
   add_menu_separator();
-  add_menu_item(STRADD, handle_menu_add, GDK_a);
+  add_menu_item(_("Add song"), handle_menu_add, GDK_a);
+#ifdef HAVE_NFTW
+  add_menu_item(_("Add directory"), handle_menu_add_dir, GDK_d);
+#endif
   add_menu_separator();
-  add_menu_item(STRLOAD, handle_menu_load, GDK_l);
-  add_menu_item(STRSAVE, handle_menu_save, GDK_v);
-  add_menu_item(STRSORTNAME, handle_menu_sort_name, GDK_o);
-  add_menu_item(STRSORTMTIME, handle_menu_sort_mtime, GDK_t);
-  add_menu_item(STRCLEAR, handle_menu_clear, GDK_c);
+  add_menu_item(_("Load playlist"), handle_menu_load, GDK_l);
+  add_menu_item(_("Save playlist"), handle_menu_save, GDK_v);
+  add_menu_item(_("Sort playlist by name"), handle_menu_sort_name, GDK_o);
+  add_menu_item(_("Sort playlist by swapped name"),
+                handle_menu_sort_swapped_name, GDK_w);
+  add_menu_item(_("Sort playlist by mtime"), handle_menu_sort_mtime, GDK_t);
+  add_menu_item(_("Clear playlist"), handle_menu_clear, GDK_c);
+  add_menu_separator();
+  add_menu_item(_("Quit"), handle_destroy, GDK_q);
   
   /* Read the config file for the first time. */
   read_config_files(0);
@@ -386,7 +412,7 @@ int main(int argc, char **argv) {
 
   /* Parse arguments. */
   while (1) {
-    char *argstring = "aslmpiwtVhoqOd1TbcWk:";
+    char *argstring = "aslmpiwtVhoqOd1TbcWk:fD";
 #ifndef NOGETOPTLONG
     static struct option long_options[] = {
       {"autostart",0,0,0}, /* -a */
@@ -409,6 +435,8 @@ int main(int argc, char **argv) {
       {"nocheckfiles",0,0,0}, /* -c */
       {"writeplaying",0,0,0}, /* -W */
       {"skippath", required_argument, 0, 0}, /* -k */
+      {"commandlinesongs",0,0,0}, /* -f */
+      {"commandlinedirs",0,0,0}, /* -D */
       {0,0,0,0}
     };
     j = 0;
@@ -465,7 +493,7 @@ int main(int argc, char **argv) {
       mode_hide_player_output = 1;
       break;
     case 'O': /* -O: sort on load */
-      mode_sort_on_load = 1;
+      attribute_sort_on_load = SORT_NAME;
       break;
     case 'd': /* -d: demangle names */
       mode_demangle_names = 1;
@@ -488,49 +516,64 @@ int main(int argc, char **argv) {
     case 'k': /* -k: skip path */
       mode_skip_path = atoi(optarg);
       break;
+    case 'f': /* -f: songs (rather than playlist) on command line */
+      mode_commandline_songs = 1;
+      break;
+    case 'D': /* -d: directories (rather than playlists) on command line */
+      mode_commandline_dirs = 1;
+      break;
     case 'V':
       print_version();
       exit(0);
       break;
     case 'h':
       print_version();
-      g_print("\nUsage: xhippo [options] [playlist [playlist ...]]
--a  --autostart    Start playing automatically once the playlists have
-                   been loaded.
--s  --scroll       Automatically scroll the list when a random song
-                   is picked.
--l  --leftscroll   Place scrollbar on the left side of the window.
--m  --mustplayall  When picking a random song, pick one that has not
-                   already been played if possible.
--1  --onetime      Stop once the entire list has been played.
--p  --showpid      Show the PIDs of player processes in the status bar.
--i  --readid3      Attempt to read ID3 tags from files and use the titles
-                   stored in the list.
--S  --stripextension
-                   Strip extensions from files shown in the playlist.
--d  --demanglenames
-                   Replace underscores and %%20s in names with spaces.
--w  --savewinstate Save the window position and state upon exit to
-                   $HOME/.xhippo/winstate.
--T  --playlisttitle
-                   Make the window title the name of the loaded playlist.
--b  --titlebasename
-                   If -T is set, use the basename of the playlist.
--t  --startmini    Force xhippo to start with its window in the \"mini\"
-                   state (with the list hidden).
--q  --hideplayeroutput
-                   Redirect player stdout and stderr to /dev/null.
--O  --sortonload   Sort the playlist immediately after loading it.
--W  --writeplaying Write the name of the currently-playing song to
-                   $HOME/.xhippo/current_song.
--V  --version      Print xhippo's version and exit.
--c  --nocheckfiles Don't bother checking whether the files in the playlist
-                   actually exist.
--o  --ordered      Play the files in the order specified in the playlist,
-                   rather than in random order.
--h  --help         This help.
-
-Report bugs to bug-xhippo@gnu.org.\n");
+      g_print("\nUsage: xhippo [options] [playlist [playlist ...]]\n"
+"or:    xhippo -f [options] [song [song ...]]\n"
+"-a  --autostart    Start playing automatically once the playlists have\n"
+"                   been loaded.\n"
+"-s  --scroll       Automatically scroll the list when a random song\n"
+"                   is picked.\n"
+"-l  --leftscroll   Place scrollbar on the left side of the window.\n"
+"-m  --mustplayall  When picking a random song, pick one that has not\n"
+"                   already been played if possible.\n"
+"-1  --onetime      Stop once the entire list has been played.\n"
+"-p  --showpid      Show the PIDs of player processes in the status bar.\n"
+"-i  --readid3      Attempt to read ID3 tags from files and use the titles\n"
+"                   stored in the list.\n"
+"-S  --stripextension\n"
+"                   Strip extensions from files shown in the playlist.\n"
+"-d  --demanglenames\n"
+"                   Replace underscores and %%20s in names with spaces.\n"
+"-w  --savewinstate Save the window position and state upon exit to\n"
+"                   $HOME/.xhippo/winstate.\n"
+"-T  --playlisttitle\n"
+"                   Make the window title the name of the loaded playlist.\n"
+"-b  --titlebasename\n"
+"                   If -T is set, use the basename of the playlist.\n"
+"-t  --startmini    Force xhippo to start with its window in the \"mini\"\n"
+"                   state (with the list hidden).\n"
+"-q  --hideplayeroutput\n"
+"                   Redirect player stdout and stderr to /dev/null.\n"
+"-O  --sortonload   Sort the playlist by name immediately after loading it.\n"
+"-W  --writeplaying Write the name of the currently-playing song to\n"
+"                   $HOME/.xhippo/current_song.\n"
+"-V  --version      Print xhippo's version and exit.\n"
+"-c  --nocheckfiles Don't bother checking whether the files in the playlist\n"
+"                   actually exist.\n"
+"-o  --ordered      Play the files in the order specified in the playlist,\n"
+"                   rather than in random order.\n"
+"-k  --skippath <n> Skip <n> elements of the path of files and display the rest.\n"
+"-f  --commandlinesongs\n"
+"                   Take the non-option arguments as songs to add to the\n"
+"                   playlist rather than playlists to load.\n"
+"-D  --commandlinedirs\n"
+"                   Take the non-option arguments as directories to search\n"
+"                   for files to add to the playlist rather than playlists\n"
+"                   to load.\n"
+"-h  --help         This help.\n"
+"\n"
+"Report bugs to bug-xhippo@gnu.org.\n");
       exit(0);
       break;
     default:
@@ -551,7 +594,7 @@ Report bugs to bug-xhippo@gnu.org.\n");
   gtk_signal_connect(GTK_OBJECT(filelist), "select_row",
 		     GTK_SIGNAL_FUNC(handle_list_select), NULL);
   gtk_signal_connect(GTK_OBJECT(filelist), "button_press_event",
-		     GTK_SIGNAL_FUNC(handle_list_click), NULL);
+		     GTK_SIGNAL_FUNC(handle_menu_popup), NULL);
   gtk_widget_set_name(filelist, "filelist");
 
   vadj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 0, 0, 0, 0);
@@ -568,7 +611,13 @@ Report bugs to bug-xhippo@gnu.org.\n");
 
   /* Deal with non-options */
   while (optind < argc) {
-    read_playlist(argv[optind++]);
+    if (mode_commandline_songs) {
+      add_file(argv[optind++]);
+    } else if (mode_commandline_dirs) {
+      add_directory(argv[optind++]);
+    } else {
+      read_playlist(argv[optind++]);
+    }
   }
   
   /* Show the list and scrollbar. */
@@ -589,7 +638,7 @@ Report bugs to bug-xhippo@gnu.org.\n");
   read_config_files(1);
 
   /* Set the status. */
-  snprintf(buf, STRBUFSIZE, STRNUMFILES, listcount);
+  snprintf(buf, STRBUFSIZE, _("%d files in list."), listcount);
   status(buf);
 
   /* Read the window attributes as appropriate. */
